"""
A Quasar is a radio emitting active galactic nucleus (i.e. an actively feeding supermassive black hole).
Radio Astronomy for the win!!!!
"""
import argparse
import logging
import os
import pathlib
import sys

import glob
import re

from pycapo import CapoConfig

from .interfaces import CasaEmulatorIF
from .forger import forge

"""
The Vela system allows for testing Workspaces workflows without submitting to CASA on the cluster.
"""

logger = logging.getLogger("vela")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class CalibrationEmulator(CasaEmulatorIF):
    def __init__(self, metadata: str,  ppr: str):
        self.ppr = ppr
        self.metadata = metadata

    def run(self):
        logger.info("Vela waiting on the Forger....")
        forge(self.metadata)

    def setup_environment(self, parameters: dict):
        os.environ['SCIPIPE_ROOTDIR'] = parameters["rootDirectory"]
        os.environ['CASA_HOME'] = parameters["homeForReprocessing"]
        os.environ['PPR_FILENAME'] = str(self.ppr)
        os.environ['LANG'] = "en_US.UTF-8"

        logger.info("Checking processing environment:")
        env1 = os.environ.get('SCIPIPE_ROOTDIR')
        logger.info(f"SCIPIPE_ROOTDIR: {env1}")
        env2 = os.environ.get('CASA_HOME')
        logger.info(f"CASA_HOME: {env2}")
        env3 = os.environ.get('PPR_FILENAME')
        logger.info(f"PPR_FILENAME: {env3}")
        env4 = os.environ.get('LANG')
        logger.info(f"LANG: {env4}")
        if 'None' in [env1, env2, env3, env4]:
            logger.info("Environment setup Failed!")
            sys.exit(1)
        else:
            logger.info("Environment ready for processing")

    def check_logs(self, parent_path: str):
        logger.info("CHECKING VELA LOGS!")

        if not os.getcwd().endswith("/working"):
            os.chdir(parent_path + "/working")

        vela_logs = glob.glob('vela-*.log')

        for file in vela_logs:
            if re.match("^.*SEVERE\sflagmanager.*$", open(file).read()):
                logger.info("VELA ERROR! Please inspect logs.")
            else:
                logger.info("VELA Success!")


def _get_settings(cwd: pathlib.Path):
    use_casa = CapoConfig().getboolean("edu.nrao.archive.workspaces.ProcessingSettings.useCasa")
    casa_home = CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions").homeForReprocessing

    root_dir = str(cwd.parent)
    processing_dir = str(cwd.stem)

    return {
        "useCasa": use_casa,
        "homeForReprocessing": casa_home,
        "rootDirectory": root_dir,
        "processingDirectory": processing_dir
    }


def parser() -> argparse.ArgumentParser:
    arg_parser = argparse.ArgumentParser(
        description="Workspaces VELA processing launcher",
        formatter_class=argparse.RawTextHelpFormatter
    )
    arg_parser.add_argument(
        "--standard-cal",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration pipeline"
    )
    arg_parser.add_argument(
        "--standard-image",
        nargs=2,
        action="store",
        required=False,
        help="run the standard imaging pipeline"
    )
    return arg_parser


def main():
    args = parser().parse_args()
    metadata = args.standard_cal[0]
    ppr = args.standard_cal[1]

    path = os.getcwd()
    settings = _get_settings(pathlib.Path(path))
    if args.standard_cal:
        emulator = CalibrationEmulator(metadata, ppr)
        emulator.setup_environment(settings)
        emulator.run()
        emulator.check_logs(parent_path=path)
        os.chdir(path)
    else:
        logger.info("ARGUMENT ERROR: no valid argument was provided.")
